const join = require('path').join
const fs = require('fs')
const fm = require('front-matter')
const pathToRegexp = require('path-to-regexp')
const paramCase = require('param-case')

const pagesData = {}

/**
 * Returns 2D array of content data, each with its respective
 * registered directory, options, and nested file data.
 *
 * The data includes both the data that will be used to create the route
 * and the data that will be injected into the component's layout.
 */
module.exports = function getPagesData (config) {
  const srcPath = config.contentPath
  config.dirs.forEach(type => { // get files in every registered directory
    const dir = join('/', type[0])
    const opts = getDirOpts(type[1] || {}, config)
    const path = join(srcPath, dir)
    const otherDirs = getOtherRegisteredDirs(dir, config.dirs)

    fs.readdirSync(path).forEach(stat => {
      const statPath = join(path, stat)

      if(fs.statSync(statPath).isDirectory()) { // Nested Files
        const dirSection = join(dir, stat)
        if (!(otherDirs.indexOf(dirSection) > -1)) {
          getFilesData(dirSection, srcPath, otherDirs, opts)
        }
      } else { // Top Level file
        pagesData[statPath] = getFileData(statPath, dir, opts)
      }

    })
  })
  return pagesData
}


/**
 * Recursively get all content files with their respective metadata.
 */
function getFilesData (contentDir, srcPath, blacklist, opts, filesData = []) {
  const contentPath = join(srcPath, contentDir)

  fs.readdirSync(contentPath).forEach(stat => {
    const statPath = join(contentPath, stat)

    if(fs.statSync(statPath).isDirectory()) {
      const nestedContentDir = join(contentDir, stat)
      if (!(blacklist.indexOf(nestedContentDir) > -1)) {
        getFilesData(nestedContentDir, srcPath, blacklist, opts, filesData)
      }
    } else {
      pagesData[statPath] = getFileData(statPath, contentDir, opts)
    }
  })
}


/**
 * Gets data required to create component's route and page layout.
 *
 * Data is retrieved either via 1) front-matter 2) file name 3) config options.
 */
function getFileData(filePath, section, opts) {
  const source = fs.readFileSync(filePath, 'utf-8')
  const metadata = fm(source).attributes

  const fileName = filePath.replace(/^.*[\\\/]/, '')
  const fileDate = fileName.match(/!?(\d{4}-\d{2}-\d{2})/)

  let permalink
  let date
  if (metadata.permalink) {
    permalink = metadata.permalink
  } else {
    const urlOpts = {
      section,
      slug: paramCase(metadata.slug || toSlug(fileName))
    }

    if (opts.isPost) {
      date = fileDate[0]
      const dateData = date.split('-')
      urlOpts.year = dateData[0]
      urlOpts.month = dateData[1]
      urlOpts.day = dateData[2]
    }

    const toPath = pathToRegexp.compile(opts.permalink)
    permalink = toPath(urlOpts, { pretty: true })
      .replace(/%2F/gi, "/") // make url encoded slash pretty
  }

  const pageData = Object.assign({}, opts.data, metadata, {
    date,
    permalink,
    section
  })

  return {
    options: opts,
    page: pageData,
    route: {
      path: join("/", permalink),
      component: filePath
    }
  }
}


/**
 * Gets content options via 1) directory options 2) global content config.
 */
function getDirOpts (dir, config) {
  const mergedData = Object.assign({}, config.data, dir.data)
  const opts = Object.assign({}, config, dir, { data: mergedData })

  return {
    route: join('/', opts.route),
    permalink: opts.permalink,
    isPost: !(opts.isPost === false),
    data: opts.data
  }
}

/**
 * Gets all the registered content directory types.
 */
function  getOtherRegisteredDirs (currDir, contentTypes) {
  const dirs = []
  contentTypes.forEach(type => {
    const dir = type[0]
    if (dir !== currDir) dirs.push(join('/' + dir))
  })
  return dirs
}


/**
 * Remove date and ext from file name.
 */
function toSlug (fileName) {
  const date = /!?(\d{4}-\d{2}-\d{2}-)/
  const ext = /(.)[^.]+$/
  return fileName.replace(date, '').replace(ext, '')
}
